@@ -35,6 +35,10 @@ import sys
 import threading
 import weakref
 
+is_win32 = sys.platform == 'win32'
+if is_win32:
+    import msvcrt
+
 try:
     import __builtin__
 except ImportError:
@@ -504,18 +508,7 @@ def _mvstr(mv):
     return _ffi.buffer(mv.mv_data, mv.mv_size)[:]
 
 def enable_drop_gil():
-    """
-    Arrange for the global interpreter lock to be released during database IO.
-    This flag is ignored and always assumed to be ``True`` on CFFI. Note this
-    can only be set once per process.
-
-    Continually dropping and reacquiring the GIL may incur unnecessary overhead
-    in single-threaded programs. Since Python intra-process concurrency is
-    already limited, and LMDB supports inter-process access, programs using
-    LMDB will achieve better throughput by forking rather than using threads.
-
-    *Caution:* this function should be invoked before any threads are created.
-    """
+    """Deprecated."""
 
 def version():
     """
@@ -744,6 +737,9 @@ class Environment(object):
         Equivalent to `mdb_env_copyfd()
         <http://symas.com/mdb/doc/group__mdb.html#ga5d51d6130325f7353db0955dbedbc378>`_
         """
+        if is_win32:
+            # Convert C library handle to kernel handle.
+            fd = msvcrt.get_osfhandle(fd)
         rc = _lib.mdb_env_copyfd(self._env, fd)
         if rc:
             raise _error("mdb_env_copyfd", rc)
