@@ -22,11 +22,8 @@
 
 #define PY_SSIZE_T_CLEAN
 
-#ifdef _MSC_VER
-#   include "msvc-stdint.h"
-#else
-#   include <stdint.h>
-#endif
+/* Search lib/win32 first, then fallthrough to <stdint.h> as required.*/
+#include "stdint.h"
 
 #include <errno.h>
 #include <string.h>
@@ -40,6 +37,10 @@
 #include "memsink.h"
 #endif
 
+#ifdef _WIN32
+#include <windows.h> /* HANDLE */
+#endif
+
 #include "lmdb.h"
 
 
@@ -61,6 +62,16 @@
 #endif
 
 
+/**
+ * On Win32, Environment.copyfd() needs _get_osfmodule() from the C library,
+ * except that function performs no input validation. So instead we import
+ * msvcrt standard library module, which wraps _get_osfmodule() in a way that
+ * is crash-safe.
+ */
+#ifdef _WIN32
+static PyObject *msvcrt;
+#endif
+
 /**
  * Integer indices into `string_tbl', representing an associated string. We
  * use an enum of indices since they can be stored with 1 byte instead of 8
@@ -175,8 +186,6 @@ static PyObject *py_int_max;
 static PyObject *py_size_max;
 /** lmdb.Error type. */
 static PyObject *Error;
-/** If 1, save_thread() and restore_thread() drop GIL. */
-static int drop_gil = 0;
 
 /** Typedefs and forward declarations. */
 static PyTypeObject PyDatabase_Type;
@@ -219,6 +228,11 @@ typedef struct TransObject TransObject;
 #       define Py_TYPE(ob) (((PyObject*)(ob))->ob_type)
 #   endif
 
+#   ifndef PyVarObject_HEAD_INIT
+#       define PyVarObject_HEAD_INIT(x, y) \
+            PyObject_HEAD_INIT(x) y,
+#   endif
+
 #endif
 
 #if (PY_MAJOR_VERSION == 2) && (PY_MINOR_VERSION < 6)
@@ -596,34 +610,10 @@ val_from_buffer(MDB_val *val, PyObject *buf)
 /* Concurrency control */
 /* ------------------- */
 
-static NOINLINE PyThreadState *save_thread(void)
-{
-    PyThreadState *s = NULL;
-    if(drop_gil) {
-        s = PyEval_SaveThread();
-    }
-    return s;
-}
-
-static NOINLINE void restore_thread(PyThreadState *state)
-{
-    if(drop_gil) {
-        PyEval_RestoreThread(state);
-    }
-}
-
-/* Like Py_BEGIN_ALLOW_THREADS */
-#define DROP_GIL \
-    { PyThreadState *_save; _save = save_thread();
-
-/* Like Py_END_ALLOW_THREADS */
-#define LOCK_GIL \
-    restore_thread(_save); }
-
 #define UNLOCKED(out, e) \
-    DROP_GIL \
+    Py_BEGIN_ALLOW_THREADS \
     out = (e); \
-    LOCK_GIL
+    Py_END_ALLOW_THREADS
 
 
 /* ---------- */
@@ -816,7 +806,9 @@ parse_args(int valid, int specsize, const struct argspec *argspec,
             type_error("too many positional arguments.");
             return -1;
         }
-        size = fmin(specsize, size);
+        if(specsize < size) {
+            size = specsize;
+        }
         for(i = 0; i < size; i++) {
             if(parse_arg(argspec + i, PyTuple_GET_ITEM(args, i), out)) {
                 return -1;
@@ -999,7 +991,7 @@ db_from_name(EnvObject *env, MDB_txn *txn, const char *name,
     }
     if((rc = mdb_dbi_flags(txn, dbi, &f))) {
         err_set("mdb_dbi_flags", rc);
-        mdb_dbi_close(env, dbi);
+        mdb_dbi_close(env->env, dbi);
         return NULL;
     }
 
@@ -1036,9 +1028,9 @@ txn_db_from_name(EnvObject *env, const char *name,
     }
 
     if(! ((dbo = db_from_name(env, txn, name, flags)))) {
-        DROP_GIL
+        Py_BEGIN_ALLOW_THREADS
         mdb_txn_abort(txn);
-        LOCK_GIL
+        Py_END_ALLOW_THREADS
         return NULL;
     }
 
@@ -1068,7 +1060,7 @@ static PyObject *
 db_flags(DbObject *self, PyObject *args, PyObject *kwds)
 {
     PyObject *dct;
-    int rc;
+    unsigned int f;
 
     struct db_flags {
         TransObject *txn;
@@ -1089,7 +1081,7 @@ db_flags(DbObject *self, PyObject *args, PyObject *kwds)
     }
 
     dct = PyDict_New();
-    unsigned int f = self->flags;
+    f = self->flags;
     PyDict_SetItemString(dct, "reverse_key", py_bool(f & MDB_REVERSEKEY));
     PyDict_SetItemString(dct, "dupsort", py_bool(f & MDB_DUPSORT));
     return dct;
@@ -1111,13 +1103,34 @@ static struct PyMethodDef db_methods[] = {
 };
 
 static PyTypeObject PyDatabase_Type = {
-    PyObject_HEAD_INIT(NULL)
-    .tp_basicsize = sizeof(DbObject),
-    .tp_dealloc = (destructor) db_dealloc,
-    .tp_clear = (inquiry) db_clear,
-    .tp_flags = Py_TPFLAGS_DEFAULT,
-    .tp_methods = db_methods,
-    .tp_name = "_Database"
+    PyVarObject_HEAD_INIT(NULL, 0)
+    "_Database",                /*tp_name*/
+    sizeof(DbObject),           /*tp_basicsize*/
+    0,                          /*tp_itemsize*/
+    (destructor)db_dealloc,     /*tp_dealloc*/
+    0,                          /*tp_print*/
+    0,                          /*tp_getattr*/
+    0,                          /*tp_setattr*/
+    0,                          /*tp_compare*/
+    0,                          /*tp_repr*/
+    0,                          /*tp_as_number*/
+    0,                          /*tp_as_sequence*/
+    0,                          /*tp_as_mapping*/
+    0,                          /*tp_hash*/
+    0,                          /*tp_call*/
+    0,                          /*tp_str*/
+    0,                          /*tp_getattro*/
+    0,                          /*tp_setattro*/
+    0,                          /*tp_as_buffer*/
+    Py_TPFLAGS_DEFAULT,         /*tp_flags*/
+    0,                          /*tp_doc*/
+    0,                          /*tp_traverse*/
+    (inquiry)db_clear,          /*tp_clear*/
+    0,                          /*tp_richcompare*/
+    0,                          /*tp_weaklistoffset*/
+    0,                          /*tp_iter*/
+    0,                          /*tp_iternext*/
+    db_methods                  /*tp_methods*/
 };
 
 
@@ -1132,9 +1145,9 @@ env_clear(EnvObject *self)
     if(self->env) {
         INVALIDATE(self)
         DEBUG("Closing env")
-        DROP_GIL
+        Py_BEGIN_ALLOW_THREADS
         mdb_env_close(self->env);
-        LOCK_GIL
+        Py_END_ALLOW_THREADS
         self->env = NULL;
     }
     if(self->main_db) {
@@ -1349,9 +1362,9 @@ env_close(EnvObject *self)
         INVALIDATE(self)
         self->valid = 0;
         DEBUG("Closing env")
-        DROP_GIL
+        Py_BEGIN_ALLOW_THREADS
         mdb_env_close(self->env);
-        LOCK_GIL
+        Py_END_ALLOW_THREADS
         self->env = NULL;
     }
     Py_RETURN_NONE;
@@ -1401,6 +1414,11 @@ env_copyfd(EnvObject *self, PyObject *args)
     struct env_copyfd {
         int fd;
     } arg = {-1};
+    int rc;
+#ifdef _WIN32
+    PyObject *temp;
+    Py_ssize_t handle;
+#endif
 
     static const struct argspec argspec[] = {
         {ARG_INT, FD_S, OFFSET(env_copyfd, fd)}
@@ -1412,8 +1430,22 @@ env_copyfd(EnvObject *self, PyObject *args)
     if(arg.fd == -1) {
         return type_error("fd argument required");
     }
-    int rc;
+
+#ifdef _WIN32
+    temp = PyObject_CallMethod(msvcrt, "get_osfhandle", "i", arg.fd);
+    if(! temp) {
+        return NULL;
+    }
+    handle = PyNumber_AsSsize_t(temp, PyExc_OverflowError);
+    Py_DECREF(temp);
+    if(PyErr_Occurred()) {
+        return NULL;
+    }
+    UNLOCKED(rc, mdb_env_copyfd(self->env, (HANDLE)handle));
+#else
     UNLOCKED(rc, mdb_env_copyfd(self->env, arg.fd));
+#endif
+
     if(rc) {
         return err_set("mdb_env_copyfd", rc);
     }
@@ -1435,13 +1467,13 @@ env_info(EnvObject *self)
         {TYPE_UINT, "num_readers", offsetof(MDB_envinfo, me_numreaders)},
         {TYPE_EOF, NULL, 0}
     };
+    MDB_envinfo info;
+    int rc;
 
     if(! self->valid) {
         return err_invalid();
     }
 
-    MDB_envinfo info;
-    int rc;
     UNLOCKED(rc, mdb_env_info(self->env, &info));
     if(rc) {
         err_set("mdb_env_info", rc);
@@ -1487,10 +1519,11 @@ env_flags(EnvObject *self)
 static PyObject *
 env_max_key_size(EnvObject *self)
 {
+    int key_size;
     if(! self->valid) {
         return err_invalid();
     }
-    int key_size = mdb_env_get_maxkeysize(self->env);
+    key_size = mdb_env_get_maxkeysize(self->env);
     return PyLong_FromLongLong(key_size);
 }
 
@@ -1500,12 +1533,13 @@ env_max_key_size(EnvObject *self)
 static PyObject *
 env_max_readers(EnvObject *self)
 {
+    unsigned int readers;
+    int rc;
+
     if(! self->valid) {
         return err_invalid();
     }
-    unsigned int readers;
-    int rc = mdb_env_get_maxreaders(self->env, &readers);
-    if(rc) {
+    if((rc = mdb_env_get_maxreaders(self->env, &readers))) {
         return err_set("mdb_env_get_maxreaders", rc);
     }
     return PyLong_FromLongLong(readers);
@@ -1532,12 +1566,13 @@ env_open_db(EnvObject *self, PyObject *args, PyObject *kwds)
         {ARG_BOOL, DUPSORT_S, OFFSET(env_open_db, dupsort)},
         {ARG_BOOL, CREATE_S, OFFSET(env_open_db, create)},
     };
+    int flags;
 
     if(parse_args(self->valid, SPECSIZE(), argspec, args, kwds, &arg)) {
         return NULL;
     }
 
-    int flags = 0;
+    flags = 0;
     if(arg.reverse_key) {
         flags |= MDB_REVERSEKEY;
     }
@@ -1561,12 +1596,13 @@ env_open_db(EnvObject *self, PyObject *args, PyObject *kwds)
 static PyObject *
 env_path(EnvObject *self)
 {
+    const char *path;
+    int rc;
+
     if(! self->valid) {
         return err_invalid();
     }
 
-    const char *path;
-    int rc;
     if((rc = mdb_env_get_path(self->env, &path))) {
         return err_set("mdb_env_get_path", rc);
     }
@@ -1652,13 +1688,14 @@ env_readers(EnvObject *self)
 static PyObject *
 env_reader_check(EnvObject *self)
 {
+    int rc;
+    int dead;
+
     if(! self->valid) {
         return err_invalid();
     }
 
-    int dead;
-    int rc = mdb_reader_check(self->env, &dead);
-    if(rc) {
+    if((rc = mdb_reader_check(self->env, &dead))) {
         return err_set("mdb_reader_check", rc);
     }
     return PyLong_FromLongLong(dead);
@@ -1677,12 +1714,12 @@ env_sync(EnvObject *self, PyObject *args)
     static const struct argspec argspec[] = {
         {ARG_BOOL, FORCE_S, OFFSET(env_sync, force)}
     };
+    int rc;
 
     if(parse_args(self->valid, SPECSIZE(), argspec, args, NULL, &arg)) {
         return NULL;
     }
 
-    int rc;
     UNLOCKED(rc, mdb_env_sync(self->env, arg.force));
     if(rc) {
         return err_set("mdb_env_sync", rc);
@@ -1709,15 +1746,44 @@ static struct PyMethodDef env_methods[] = {
 };
 
 static PyTypeObject PyEnvironment_Type = {
-    PyObject_HEAD_INIT(0)
-    .tp_basicsize = sizeof(EnvObject),
-    .tp_dealloc = (destructor) env_dealloc,
-    .tp_clear = (inquiry) env_clear,
-    .tp_flags = Py_TPFLAGS_DEFAULT,
-    .tp_methods = env_methods,
-    .tp_name = "Environment",
-    .tp_new = env_new,
-    .tp_weaklistoffset = offsetof(EnvObject, weaklist),
+    PyVarObject_HEAD_INIT(NULL, 0)
+    "Environment",              /*tp_name*/
+    sizeof(EnvObject),          /*tp_basicsize*/
+    0,                          /*tp_itemsize*/
+    (destructor) env_dealloc,   /*tp_dealloc*/
+    0,                          /*tp_print*/
+    0,                          /*tp_getattr*/
+    0,                          /*tp_setattr*/
+    0,                          /*tp_compare*/
+    0,                          /*tp_repr*/
+    0,                          /*tp_as_number*/
+    0,                          /*tp_as_sequence*/
+    0,                          /*tp_as_mapping*/
+    0,                          /*tp_hash*/
+    0,                          /*tp_call*/
+    0,                          /*tp_str*/
+    0,                          /*tp_getattro*/
+    0,                          /*tp_setattro*/
+    0,                          /*tp_as_buffer*/
+    Py_TPFLAGS_DEFAULT,         /*tp_flags*/
+    0,                          /*tp_doc*/
+    0,                          /*tp_traverse*/
+    (inquiry) env_clear,        /*tp_clear*/
+    0,                          /*tp_richcompare*/
+    offsetof(EnvObject, weaklist), /*tp_weaklistoffset*/
+    0,                          /*tp_iter*/
+    0,                          /*tp_iternext*/
+    env_methods,                /*tp_methods*/
+    0,                          /*tp_members*/
+    0,                          /*tp_getset*/
+    0,                          /*tp_base*/
+    0,                          /*tp_dict*/
+    0,                          /*tp_descr_get*/
+    0,                          /*tp_descr_set*/
+    0,                          /*tp_dictoffset*/
+    0,                          /*tp_init*/
+    0,                          /*tp_alloc*/
+    env_new,                    /*tp_new*/
 };
 
 
@@ -1731,9 +1797,9 @@ cursor_clear(CursorObject *self)
     if(self->valid) {
         INVALIDATE(self)
         UNLINK_CHILD(self->trans, self)
-        DROP_GIL
+        Py_BEGIN_ALLOW_THREADS
         mdb_cursor_close(self->curs);
-        LOCK_GIL
+        Py_END_ALLOW_THREADS
         self->valid = 0;
     }
     Py_CLEAR(self->trans);
@@ -1783,12 +1849,13 @@ cursor_new(PyTypeObject *type, PyObject *args, PyObject *kwds)
 static PyObject *
 cursor_count(CursorObject *self)
 {
+    size_t count;
+    int rc;
+
     if(! self->valid) {
         return err_invalid();
     }
 
-    size_t count;
-    int rc;
     UNLOCKED(rc, mdb_cursor_count(self->curs, &count));
     if(rc) {
         return err_set("mdb_cursor_count", rc);
@@ -1852,18 +1919,19 @@ cursor_delete(CursorObject *self, PyObject *args, PyObject *kwds)
     static const struct argspec argspec[] = {
         {ARG_BOOL, DUPDATA_S, OFFSET(cursor_delete, dupdata)}
     };
+    int res;
 
     if(parse_args(self->valid, SPECSIZE(), argspec, args, kwds, &arg)) {
         return NULL;
     }
 
-    int res = 0;
+    res = 0;
     if(self->positioned) {
+        int rc;
+        int flags = arg.dupdata ? MDB_NODUPDATA : 0;
         DEBUG("deleting key '%.*s'",
               (int) self->key.mv_size,
               (char*) self->key.mv_data)
-        int rc;
-        int flags = arg.dupdata ? MDB_NODUPDATA : 0;
         UNLOCKED(rc, mdb_cursor_del(self->curs, flags));
         self->trans->mutations++;
         if(rc) {
@@ -1902,10 +1970,6 @@ cursor_value(CursorObject *self);
 static PyObject *
 cursor_get(CursorObject *self, PyObject *args, PyObject *kwds)
 {
-    if(! self->valid) {
-        return err_invalid();
-    }
-
     struct cursor_get {
         MDB_val key;
         PyObject *default_;
@@ -1941,6 +2005,11 @@ cursor_get(CursorObject *self, PyObject *args, PyObject *kwds)
 static PyObject *
 cursor_item(CursorObject *self)
 {
+    int as_buffer;
+    PyObject *key;
+    PyObject *val;
+    PyObject *tup;
+
     if(! self->valid) {
         return err_invalid();
     }
@@ -1950,10 +2019,10 @@ cursor_item(CursorObject *self)
         return NULL;
     }
 
-    int as_buffer = self->trans->flags & TRANS_BUFFERS;
-    PyObject *key = obj_from_val(&self->key, as_buffer);
-    PyObject *val = obj_from_val(&self->val, as_buffer);
-    PyObject *tup = PyTuple_New(2);
+    as_buffer = self->trans->flags & TRANS_BUFFERS;
+    key = obj_from_val(&self->key, as_buffer);
+    val = obj_from_val(&self->val, as_buffer);
+    tup = PyTuple_New(2);
     if(tup && key && val) {
         PyTuple_SET_ITEM(tup, 0, key);
         PyTuple_SET_ITEM(tup, 1, val);
@@ -2075,12 +2144,14 @@ cursor_put(CursorObject *self, PyObject *args, PyObject *kwds)
         {ARG_BOOL, OVERWRITE_S, OFFSET(cursor_put, overwrite)},
         {ARG_BOOL, APPEND_S, OFFSET(cursor_put, append)}
     };
+    int flags;
+    int rc;
 
     if(parse_args(self->valid, SPECSIZE(), argspec, args, kwds, &arg)) {
         return NULL;
     }
 
-    int flags = 0;
+    flags = 0;
     if(! arg.dupdata) {
         flags |= MDB_NODUPDATA;
     }
@@ -2091,7 +2162,6 @@ cursor_put(CursorObject *self, PyObject *args, PyObject *kwds)
         flags |= MDB_APPEND;
     }
 
-    int rc;
     UNLOCKED(rc, mdb_cursor_put(self->curs, &arg.key, &arg.val, flags));
     self->trans->mutations++;
     if(rc) {
@@ -2192,6 +2262,8 @@ cursor_pop(CursorObject *self, PyObject *args, PyObject *kwds)
     static const struct argspec argspec[] = {
         {ARG_BUF, KEY_S, OFFSET(cursor_pop, key)},
     };
+    PyObject *old;
+    int rc;
 
     if(parse_args(self->valid, SPECSIZE(), argspec, args, kwds, &arg)) {
         return NULL;
@@ -2204,12 +2276,10 @@ cursor_pop(CursorObject *self, PyObject *args, PyObject *kwds)
     if(! self->positioned) {
         Py_RETURN_NONE;
     }
-    PyObject *old = obj_from_val(&self->val, 0);
-    if(! old) {
+    if(! ((old = obj_from_val(&self->val, 0)))) {
         return NULL;
     }
 
-    int rc;
     UNLOCKED(rc, mdb_cursor_del(self->curs, 0));
     self->trans->mutations++;
     if(rc) {
@@ -2345,6 +2415,7 @@ iter_from_args(CursorObject *self, PyObject *args, PyObject *kwds,
         {ARG_BOOL, KEYS_S, OFFSET(iter_from_args, keys)},
         {ARG_BOOL, VALUES_S, OFFSET(iter_from_args, values)}
     };
+    void *val_func;
 
     if(parse_args(self->valid, SPECSIZE(), argspec, args, kwds, &arg)) {
         return NULL;
@@ -2356,7 +2427,6 @@ iter_from_args(CursorObject *self, PyObject *args, PyObject *kwds,
         }
     }
 
-    void *val_func;
     if(! arg.values) {
         val_func = cursor_key;
     } else if(! arg.keys) {
@@ -2442,12 +2512,13 @@ cursor_iter_from(CursorObject *self, PyObject *args)
         {ARG_BUF, KEY_S, OFFSET(cursor_iter_from, key)},
         {ARG_BOOL, REVERSE_S, OFFSET(cursor_iter_from, reverse)}
     };
+    enum MDB_cursor_op op;
+    int rc;
 
     if(parse_args(self->valid, SPECSIZE(), argspec, args, NULL, &arg)) {
         return NULL;
     }
 
-    int rc;
     if((! arg.key.mv_size) && (! arg.reverse)) {
         rc = _cursor_get_c(self, MDB_FIRST);
     } else {
@@ -2459,7 +2530,7 @@ cursor_iter_from(CursorObject *self, PyObject *args)
         return NULL;
     }
 
-    enum MDB_cursor_op op = MDB_NEXT;
+    op = MDB_NEXT;
     if(arg.reverse) {
         op = MDB_PREV;
         if(! self->positioned) {
@@ -2508,15 +2579,45 @@ static struct PyMethodDef cursor_methods[] = {
 };
 
 static PyTypeObject PyCursor_Type = {
-    PyObject_HEAD_INIT(0)
-    .tp_basicsize = sizeof(CursorObject),
-    .tp_dealloc = (destructor) cursor_dealloc,
-    .tp_clear = (inquiry) cursor_clear,
-    .tp_flags = Py_TPFLAGS_DEFAULT,
-    .tp_iter = (getiterfunc)cursor_iter,
-    .tp_methods = cursor_methods,
-    .tp_name = "Cursor",
-    .tp_new = cursor_new
+    PyVarObject_HEAD_INIT(NULL, 0)
+    "Cursor",                   /*tp_name*/
+    sizeof(CursorObject),       /*tp_basicsize*/
+    0,                          /*tp_itemsize*/
+    (destructor) cursor_dealloc,/*tp_dealloc*/
+    0,                          /*tp_print*/
+    0,                          /*tp_getattr*/
+    0,                          /*tp_setattr*/
+    0,                          /*tp_compare*/
+    0,                          /*tp_repr*/
+    0,                          /*tp_as_number*/
+    0,                          /*tp_as_sequence*/
+    0,                          /*tp_as_mapping*/
+    0,                          /*tp_hash*/
+    0,                          /*tp_call*/
+    0,                          /*tp_str*/
+    0,                          /*tp_getattro*/
+    0,                          /*tp_setattro*/
+    0,                          /*tp_as_buffer*/
+    Py_TPFLAGS_DEFAULT,         /*tp_flags*/
+    0,                          /*tp_doc*/
+    0,                          /*tp_traverse*/
+    (inquiry) cursor_clear,     /*tp_clear*/
+    0,                          /*tp_richcompare*/
+    0,                          /*tp_weaklistoffset*/
+    (getiterfunc)cursor_iter,   /*tp_iter*/
+    0,                          /*tp_iternext*/
+    cursor_methods,             /*tp_methods*/
+    0,                          /*tp_members*/
+    0,                          /*tp_getset*/
+    0,                          /*tp_base*/
+    0,                          /*tp_dict*/
+    0,                          /*tp_descr_get*/
+    0,                          /*tp_descr_set*/
+    0,                          /*tp_dictoffset*/
+    0,                          /*tp_init*/
+    0,                          /*tp_alloc*/
+    cursor_new,                 /*tp_new*/
+    0,                          /*tp_free*/
 };
 
 
@@ -2558,6 +2659,7 @@ iter_next(IterObject *self)
     if(! self->curs->positioned) {
         return NULL;
     }
+
     if(self->started) {
         if(_cursor_get_c(self->curs, self->op)) {
             return NULL;
@@ -2566,9 +2668,9 @@ iter_next(IterObject *self)
             return NULL;
         }
     }
-    PyObject *val = self->val_func(self->curs);
+
     self->started = 1;
-    return val;
+    return self->val_func(self->curs);
 }
 
 static struct PyMethodDef iter_methods[] = {
@@ -2577,17 +2679,49 @@ static struct PyMethodDef iter_methods[] = {
 };
 
 static PyTypeObject PyIterator_Type = {
-    PyObject_HEAD_INIT(0)
-    .tp_basicsize = sizeof(IterObject),
-    .tp_dealloc = (destructor) iter_dealloc,
-    .tp_flags = Py_TPFLAGS_DEFAULT,
-    .tp_iter = (getiterfunc)iter_iter,
-    .tp_iternext = (iternextfunc)iter_next,
-    .tp_methods = iter_methods,
-    .tp_name = "Iterator"
+    PyVarObject_HEAD_INIT(NULL, 0)
+    "Iterator",                 /*tp_name*/
+    sizeof(IterObject),         /*tp_basicsize*/
+    0,                          /*tp_itemsize*/
+    (destructor) iter_dealloc,  /*tp_dealloc*/
+    0,                          /*tp_print*/
+    0,                          /*tp_getattr*/
+    0,                          /*tp_setattr*/
+    0,                          /*tp_compare*/
+    0,                          /*tp_repr*/
+    0,                          /*tp_as_number*/
+    0,                          /*tp_as_sequence*/
+    0,                          /*tp_as_mapping*/
+    0,                          /*tp_hash*/
+    0,                          /*tp_call*/
+    0,                          /*tp_str*/
+    0,                          /*tp_getattro*/
+    0,                          /*tp_setattro*/
+    0,                          /*tp_as_buffer*/
+    Py_TPFLAGS_DEFAULT,         /*tp_flags*/
+    0,                          /*tp_doc*/
+    0,                          /*tp_traverse*/
+    0,                          /*tp_clear*/
+    0,                          /*tp_richcompare*/
+    0,                          /*tp_weaklistoffset*/
+    (getiterfunc)iter_iter,     /*tp_iter*/
+    (iternextfunc)iter_next,    /*tp_iternext*/
+    iter_methods,               /*tp_methods*/
+    0,                          /*tp_members*/
+    0,                          /*tp_getset*/
+    0,                          /*tp_base*/
+    0,                          /*tp_dict*/
+    0,                          /*tp_descr_get*/
+    0,                          /*tp_descr_set*/
+    0,                          /*tp_dictoffset*/
+    0,                          /*tp_init*/
+    0,                          /*tp_alloc*/
+    0,                          /*tp_new*/
+    0,                          /*tp_free*/
 };
 
 
+
 /* ------------ */
 /* Transactions */
 /* ------------ */
@@ -2602,9 +2736,9 @@ trans_clear(TransObject *self)
 #endif
         if(self->txn) {
             DEBUG("aborting")
-            DROP_GIL
+            Py_BEGIN_ALLOW_THREADS
             mdb_txn_abort(self->txn);
-            LOCK_GIL
+            Py_END_ALLOW_THREADS
             self->txn = NULL;
         }
         Py_CLEAR(self->db);
@@ -2669,9 +2803,9 @@ trans_abort(TransObject *self)
 #ifdef HAVE_MEMSINK
         ms_notify((PyObject *) self, &self->sink_head);
 #endif
-        DROP_GIL
+        Py_BEGIN_ALLOW_THREADS
         mdb_txn_abort(self->txn);
-        LOCK_GIL
+        Py_END_ALLOW_THREADS
         self->txn = NULL;
         self->valid = 0;
     }
@@ -2684,6 +2818,8 @@ trans_abort(TransObject *self)
 static PyObject *
 trans_commit(TransObject *self)
 {
+    int rc;
+
     if(! self->valid) {
         return err_invalid();
     }
@@ -2692,7 +2828,6 @@ trans_commit(TransObject *self)
 #ifdef HAVE_MEMSINK
     ms_notify((PyObject *) self, &self->sink_head);
 #endif
-    int rc;
     UNLOCKED(rc, mdb_txn_commit(self->txn));
     self->txn = NULL;
     self->valid = 0;
@@ -2739,6 +2874,8 @@ trans_delete(TransObject *self, PyObject *args, PyObject *kwds)
         {ARG_BUF, VALUE_S, OFFSET(trans_delete, val)},
         {ARG_DB, DB_S, OFFSET(trans_delete, db)}
     };
+    MDB_val *val_ptr;
+    int rc;
 
     if(parse_args(self->valid, SPECSIZE(), argspec, args, kwds, &arg)) {
         return NULL;
@@ -2746,9 +2883,8 @@ trans_delete(TransObject *self, PyObject *args, PyObject *kwds)
     if(! db_owner_check(arg.db, self->env)) {
         return NULL;
     }
-    MDB_val *val_ptr = arg.val.mv_size ? &arg.val : NULL;
+    val_ptr = arg.val.mv_size ? &arg.val : NULL;
     self->mutations++;
-    int rc;
     UNLOCKED(rc, mdb_del(self->txn, arg.db->dbi, &arg.key, val_ptr));
     if(rc) {
         if(rc == MDB_NOTFOUND) {
@@ -2774,6 +2910,7 @@ trans_drop(TransObject *self, PyObject *args, PyObject *kwds)
         {ARG_DB, DB_S, OFFSET(trans_drop, db)},
         {ARG_BOOL, DELETE_S, OFFSET(trans_drop, delete)}
     };
+    int rc;
 
     if(parse_args(self->valid, SPECSIZE(), argspec, args, kwds, &arg)) {
         return NULL;
@@ -2784,7 +2921,6 @@ trans_drop(TransObject *self, PyObject *args, PyObject *kwds)
         return NULL;
     }
 
-    int rc;
     UNLOCKED(rc, mdb_drop(self->txn, arg.db->dbi, arg.delete));
     self->mutations++;
     if(rc) {
@@ -2810,6 +2946,8 @@ trans_get(TransObject *self, PyObject *args, PyObject *kwds)
         {ARG_OBJ, DEFAULT_S, OFFSET(trans_get, default_)},
         {ARG_DB, DB_S, OFFSET(trans_get, db)}
     };
+    MDB_val val;
+    int rc;
 
     if(parse_args(self->valid, SPECSIZE(), argspec, args, kwds, &arg)) {
         return NULL;
@@ -2822,8 +2960,6 @@ trans_get(TransObject *self, PyObject *args, PyObject *kwds)
         return type_error("key must be given.");
     }
 
-    MDB_val val;
-    int rc;
     UNLOCKED(rc, mdb_get(self->txn, arg.db->dbi, &arg.key, &val));
     if(rc) {
         if(rc == MDB_NOTFOUND) {
@@ -2858,6 +2994,8 @@ trans_put(TransObject *self, PyObject *args, PyObject *kwds)
         {ARG_BOOL, APPEND_S, OFFSET(trans_put, append)},
         {ARG_DB, DB_S, OFFSET(trans_put, db)}
     };
+    int flags;
+    int rc;
 
     if(parse_args(self->valid, SPECSIZE(), argspec, args, kwds, &arg)) {
         return NULL;
@@ -2866,7 +3004,7 @@ trans_put(TransObject *self, PyObject *args, PyObject *kwds)
         return NULL;
     }
 
-    int flags = 0;
+    flags = 0;
     if(! arg.dupdata) {
         flags |= MDB_NODUPDATA;
     }
@@ -2884,7 +3022,6 @@ trans_put(TransObject *self, PyObject *args, PyObject *kwds)
         (int)arg.value.mv_size)
 
     self->mutations++;
-    int rc;
     UNLOCKED(rc, mdb_put(self->txn, (arg.db)->dbi,
                          &arg.key, &arg.value, flags));
     if(rc) {
@@ -2918,6 +3055,8 @@ trans_replace(TransObject *self, PyObject *args, PyObject *kwds)
         {ARG_BUF, VALUE_S, OFFSET(trans_replace, value)},
         {ARG_DB, DB_S, OFFSET(trans_replace, db)}
     };
+    PyObject *ret;
+    CursorObject *cursor;
 
     if(parse_args(self->valid, SPECSIZE(), argspec, args, kwds, &arg)) {
         return NULL;
@@ -2926,8 +3065,8 @@ trans_replace(TransObject *self, PyObject *args, PyObject *kwds)
         return NULL;
     }
 
-    PyObject *ret = NULL;
-    CursorObject *cursor = (CursorObject *) make_cursor(arg.db, self);
+    ret = NULL;
+    cursor = (CursorObject *) make_cursor(arg.db, self);
     if(cursor) {
         ret = do_cursor_replace(cursor, &arg.key, &arg.value);
         Py_DECREF(cursor);
@@ -2953,6 +3092,9 @@ trans_pop(TransObject *self, PyObject *args, PyObject *kwds)
         {ARG_BUF, KEY_S, OFFSET(trans_pop, key)},
         {ARG_DB, DB_S, OFFSET(trans_pop, db)}
     };
+    CursorObject *cursor;
+    PyObject *old;
+    int rc;
 
     if(parse_args(self->valid, SPECSIZE(), argspec, args, kwds, &arg)) {
         return NULL;
@@ -2961,8 +3103,7 @@ trans_pop(TransObject *self, PyObject *args, PyObject *kwds)
         return NULL;
     }
 
-    CursorObject *cursor = (CursorObject *) make_cursor(arg.db, self);
-    if(! cursor) {
+    if(! ((cursor = (CursorObject *) make_cursor(arg.db, self)))) {
         return NULL;
     }
 
@@ -2975,13 +3116,11 @@ trans_pop(TransObject *self, PyObject *args, PyObject *kwds)
         Py_DECREF((PyObject *)cursor);
         Py_RETURN_NONE;
     }
-    PyObject *old = obj_from_val(&cursor->val, 0);
-    if(! old) {
+    if(! ((old = obj_from_val(&cursor->val, 0)))) {
         Py_DECREF((PyObject *)cursor);
         return NULL;
     }
 
-    int rc;
     UNLOCKED(rc, mdb_cursor_del(cursor->curs, 0));
     Py_DECREF((PyObject *)cursor);
     self->mutations++;
@@ -3032,6 +3171,8 @@ trans_stat(TransObject *self, PyObject *args, PyObject *kwds)
     static const struct argspec argspec[] = {
         {ARG_DB, DB_S, OFFSET(trans_stat, db)}
     };
+    MDB_stat st;
+    int rc;
 
     if(parse_args(self->valid, SPECSIZE(), argspec, args, kwds, &arg)) {
         return NULL;
@@ -3040,8 +3181,6 @@ trans_stat(TransObject *self, PyObject *args, PyObject *kwds)
         return NULL;
     }
 
-    MDB_stat st;
-    int rc;
     UNLOCKED(rc, mdb_stat(self->txn, arg.db->dbi, &st));
     if(rc) {
         return err_set("mdb_stat", rc);
@@ -3066,23 +3205,54 @@ static struct PyMethodDef trans_methods[] = {
 };
 
 static PyTypeObject PyTransaction_Type = {
-    PyObject_HEAD_INIT(0)
-    .tp_basicsize = sizeof(TransObject),
-    .tp_dealloc = (destructor) trans_dealloc,
-    .tp_clear = (inquiry) trans_clear,
-    .tp_flags = Py_TPFLAGS_DEFAULT,
-    .tp_methods = trans_methods,
-    .tp_name = "Transaction",
-    .tp_new = trans_new,
+    PyVarObject_HEAD_INIT(NULL, 0)
+    "Transaction",              /*tp_name*/
+    sizeof(TransObject),        /*tp_basicsize*/
+    0,                          /*tp_itemsize*/
+    (destructor) trans_dealloc, /*tp_dealloc*/
+    0,                          /*tp_print*/
+    0,                          /*tp_getattr*/
+    0,                          /*tp_setattr*/
+    0,                          /*tp_compare*/
+    0,                          /*tp_repr*/
+    0,                          /*tp_as_number*/
+    0,                          /*tp_as_sequence*/
+    0,                          /*tp_as_mapping*/
+    0,                          /*tp_hash*/
+    0,                          /*tp_call*/
+    0,                          /*tp_str*/
+    0,                          /*tp_getattro*/
+    0,                          /*tp_setattro*/
+    0,                          /*tp_as_buffer*/
+    Py_TPFLAGS_DEFAULT,         /*tp_flags*/
+    0,                          /*tp_doc*/
+    0,                          /*tp_traverse*/
+    (inquiry) trans_clear,      /*tp_clear*/
+    0,                          /*tp_richcompare*/
+    0,                          /*tp_weaklistoffset*/
+    0,                          /*tp_iter*/
+    0,                          /*tp_iternext*/
+    trans_methods,              /*tp_methods*/
+    0,                          /*tp_members*/
+    0,                          /*tp_getset*/
+    0,                          /*tp_base*/
+    0,                          /*tp_dict*/
+    0,                          /*tp_descr_get*/
+    0,                          /*tp_descr_set*/
+    0,                          /*tp_dictoffset*/
+    0,                          /*tp_init*/
+    0,                          /*tp_alloc*/
+    trans_new,                  /*tp_new*/
+    0,                          /*tp_free*/
 };
 
+
 /**
  * lmdb.enable_drop_gil()
  */
 static PyObject *
 enable_drop_gil(void)
 {
-    drop_gil = 1;
     Py_RETURN_NONE;
 }
 
@@ -3148,13 +3318,15 @@ static int init_types(PyObject *mod)
  */
 static int init_strings(PyObject *mod)
 {
+    const char *cur;
+    int i;
+
     string_tbl = malloc(sizeof(PyObject *) * STRING_ID_COUNT);
     if(! string_tbl) {
         return -1;
     }
 
-    const char *cur = strings;
-    int i;
+    cur = strings;
     for(i = 0; i < STRING_ID_COUNT; i++) {
         if(! ((string_tbl[i] = PyUnicode_InternFromString(cur)))) {
             return -1;
@@ -3187,6 +3359,10 @@ static int init_constants(PyObject *mod)
  */
 static int init_errors(PyObject *mod)
 {
+    size_t count;
+    char qualname[64];
+    int i;
+
     Error = PyErr_NewException("lmdb.Error", NULL, NULL);
     if(! Error) {
         return -1;
@@ -3195,21 +3371,20 @@ static int init_errors(PyObject *mod)
         return -1;
     }
 
-    size_t count = (sizeof error_map / sizeof error_map[0]);
+    count = (sizeof error_map / sizeof error_map[0]);
     error_tbl = malloc(sizeof(PyObject *) * count);
     if(! error_tbl) {
         return -1;
     }
 
-    int i;
-    char qualname[64];
     for(i = 0; i < count; i++) {
         const struct error_map *error = &error_map[i];
+        PyObject *klass;
+
         snprintf(qualname, sizeof qualname, "lmdb.%s", error->name);
         qualname[sizeof qualname - 1] = '\0';
 
-        PyObject *klass = PyErr_NewException(qualname, Error, NULL);
-        if(! klass) {
+        if(! ((klass = PyErr_NewException(qualname, Error, NULL)))) {
             return -1;
         }
 
@@ -3247,6 +3422,12 @@ MODINIT_NAME(void)
     }
 #endif
 
+#ifdef _WIN32
+    if(! ((msvcrt = PyImport_ImportModule("msvcrt")))) {
+        MOD_RETURN(NULL);
+    }
+#endif
+
     if(init_strings(mod)) {
         MOD_RETURN(NULL);
     }
